/*
Copyright (c) 2003-2004, Gregory Kuhlmann, Peter Stone, Justin Lallinger,
University of Texas at Austin
All right reserved

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the University of Texas nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

%{
#include <string.h>
#include "ClangCondition.h"
#include "ClangDirective.h"
#include "ClangTypes.h"
#include "ClangRegion.h"
#include "ClangParse.h"
#include "ClangRule.h"
#include "ClangMsg.h"

#include "ClangRuleBase.h"


int yyerror(char *s);

#define YYPARSE_PARAM ctrl
#define YYLEX_PARAM ctrl
#define YYERROR_VERBOSE

int yylex(void *lvalp, void *ctrl);

bool isEndOfToken(char c);
int lastoff=0;

%}



%union
{
	Region *reg;
	PointRegion *ptreg;
	regionList *rlist;
	double real;
	CLTeam tm;
	UnumSet *uns;
	int integer;
	Action *act;
	Condition *cnd;
	CLPlayMode md;
	CondCompOp cndop;
	conditionList *clist;
	Directive *dir;
	actionList *alist;
	directiveList *dlist;
	Rule *rl;
	CLangVar *clv;
	idList *idl;
	ruleList *rllist;


	definemsg *defm;
	
}


%token <integer> INTEGER
%token <real> DOUBLE
%token <md> PLAYMODE
%token <clv> UNDEFCS
%token <clv> UNDEFCV
%token <clv> DEFCS
%token <clv> DEFCV


%type <real> NUMBER
%type <ptreg> point
%type <tm> team
%type <ptreg> pointarith
%type <rlist> reglist
%type <reg> region
%type <act> action
%type <uns> UNUMSET
%type <uns> UNUMSET_WO_BRACES
%type <cnd> condition
%type <cnd> condcomp
%type <cndop> ccop
%type <clist> condlist
%type <dir> directive
%type <alist> actlist
%type <rl> rule
%type <dlist> dirlist
%type <rllist> rulelist
%type <idl> idlist
%type <integer> definelist
%type <defm> definecomponent
%type <defm> definercomp
%type <defm> defineccomp
%type <defm> definedcomp
%type <defm> defineacomp
%type <defm> definerulecomp


%type <ril> ruleids

// Region tokens
%token pt "pt"
%token ball "ball"
%token oppteam "opp"
%token ourteam "our"
%token rec "rec"
%token tri "tri"
%token regnull "null"
%token ureg "reg"
%token arc "arc"

// Condition tokens
%token truecond "true"
%token falsecond "false"
%token ppos "ppos"
%token bpos "bpos"
%token bowner "bowner"
%token playm "playm"
%token andcond "and"
%token orcond "or"
%token notcond "not"
%token timecond "time"
%token opp_goals "opp_goals"
%token our_goals "our_goals"
%token goal_diff "goal_diff"
%token unumcond "unum"


// Action tokens
%token pos "pos"
%token home "home"
%token mark "mark"
%token markl "markl"
%token oline "oline"
%token htype "htype"
%token pass "pass"
%token dribble "dribble"
%token clearact "clear"
%token shoot "shoot"
%token hold "hold"
%token intercept "intercept"
%token tackle "tackle"

// Directive tokens
%token dirdo "do"
%token dirdont "dont"

// CLang message tokens
%token definerule "definerule"
%token definer "definer"
%token definec "definec"
%token definea "definea"
%token definedir "defined"
%token define "define"
%token deleterule "delete"
%token ruletoken "rule"
%token direc "direc"
%token model "model"
%token allrules "all"

%token ruleon "on"
%token ruleoff "off"

%pure_parser


%%


line:
	'(' define definelist ')' {((parsestruct *)ctrl)->itemsdefined = $3; YYACCEPT;}
|	'(' ruletoken ruleids ')' {((parsestruct *)ctrl)->itemsdefined = -1;YYACCEPT;}
|	'(' deleterule allrules ')' {rulebase->DeleteRule(); YYACCEPT;}
|	'(' deleterule '(' idlist ')' ')' {rulebase->DeleteRule($4); YYACCEPT;}
|	'(' deleterule DEFCV ')' {rulebase->DeleteRule($3); YYACCEPT;}
;


ruleids:
	ruleidcomp {}
|	ruleidcomp ruleids {}
;

ruleidcomp:
	'(' ruleon allrules ')' {rulebase->ChangeRuleState(true);}
|	'(' ruleoff allrules ')' {rulebase->ChangeRuleState(false);}
|	'(' ruleon DEFCV ')' {rulebase->ChangeRuleState($3,true);}
|	'(' ruleoff DEFCV ')' {rulebase->ChangeRuleState($3,false);}
|	'(' ruleon '(' idlist ')' ')' {rulebase->ChangeRuleState($4,true);}
|	'(' ruleoff '(' idlist ')' ')' {rulebase->ChangeRuleState($4,false);}
;



definelist:
	definelist definecomponent	{$$ = $1+1; rulebase->HearDefineMsg($2);}
|	definecomponent		{$$=1;rulebase->HearDefineMsg($1);}
;

definecomponent:
	definercomp
|	defineccomp
|	definedcomp
|	defineacomp
|	definerulecomp
;

definercomp:
	'(' definer UNDEFCS region ')' {$$ = new definemsg; $$->name = $3; $$->reg = $4; $$->type = DEFR;}
|	'(' definer DEFCS region ')' 
	{
		if (rulebase->IsPointRegion($3) && !$4->IsPoint())
		{
			char estr[255];
			sprintf(estr,"Warning: Attempting to redefine point region %s as non-point region",$3->GetName());
			yyerror(estr);
		}
		$$ = new definemsg; 
		$$->name = $3; 
		$$->reg = $4; 
		$$->type = DEFR;
	}
;

defineccomp:
	'(' definec UNDEFCS condition ')' {$$ = new definemsg; $$->name = $3; $$->cond = $4; $$->type = DEFC;}
|	'(' definec DEFCS condition ')' {$$ = new definemsg; $$->name = $3; $$->cond = $4; $$->type = DEFC;}
;

definedcomp:
	'(' definedir UNDEFCS directive ')' {$$ = new definemsg; $$->name = $3; $$->dir = $4; $$->type = DEFD;}
|	'(' definedir DEFCS directive ')' {$$ = new definemsg; $$->name = $3; $$->dir = $4; $$->type = DEFD;}
;

defineacomp:
	'(' definea UNDEFCS action ')' {$$ = new definemsg; $$->name = $3; $$->act = $4; $$->type = DEFA;}
|	'(' definea DEFCS action ')' {$$ = new definemsg; $$->name = $3; $$->act = $4; $$->type = DEFA;}
;

definerulecomp:
	'(' definerule UNDEFCV model rule ')' {$$ = new definemsg; $$->name = $3; $$->rl = $5; $$->type = DEFRL;$$->rlon = 0;  $$->rldirecmode = false;}
|	'(' definerule DEFCV model rule ')' {$$ = new definemsg; $$->name = $3; $$->rl = $5; $$->type = DEFRL;$$->rlon = 0;  $$->rldirecmode = false;}
|	'(' definerule UNDEFCV direc rule ')' {$$ = new definemsg; $$->name = $3; $$->rl = $5; $$->type = DEFRL; $$->rlon = 0;  $$->rldirecmode = true;}
|	'(' definerule DEFCV direc rule ')' {$$ = new definemsg; $$->name = $3; $$->rl = $5; $$->type = DEFRL; $$->rlon = 0;  $$->rldirecmode = true;}
;



// All rule stuff below
rule:
	'(' condition dirlist ')' {$$ = new CondDirRule($2,$3);}
|	'(' condition rulelist ')'	{$$ = new CondRuleRule($2,$3);}
|	'(' idlist ')'	{$$ = new IDListRule($2);}
|	DEFCV	{idList *l = new idList; l->r = $1; l->next = NULL; $$ = new IDListRule(l);}
;

rulelist:
	rule	{$$ = new ruleList; $$->r = $1; $$->next = NULL;}
|	rule rulelist {$$ = new ruleList; $$->r= $1; $$->next =$2;}
;


idlist:
	DEFCV	{$$ = new idList; $$->r = $1; $$->next = NULL;}
|	DEFCV idlist {$$ = new idList; $$->r = $1; $$->next = $2;}
;


// All directive stuff below

dirlist:
	directive		{$$ = new directiveList; $$->next = NULL; $$->dir = $1;}
|	directive dirlist	{$$ = new directiveList; $$->next = $2; $$->dir = $1;}
;


directive:
	'(' dirdo team UNUMSET actlist ')' {$$ = new StdDirective($5, $3, $4, true);}
|	'(' dirdont team UNUMSET actlist ')' {$$ = new StdDirective($5, $3, $4, false);}
|	DEFCS	{
			if (!rulebase->IsDirective($1))
			{
				char estr[255];
				sprintf(estr,"%s is not defined as a directive", $1->GetName());
				yyerror(estr);
				YYERROR; 
			}
			$$ = new VarDirective($1);
		}
;

actlist:
	action	{$$ = new actionList; $$->next = NULL; $$->act = $1;}
|	action actlist {$$ = new actionList; $$->next = $2; $$->act = $1;}
;

// All condition related stuff below

condition:
	'(' truecond ')' {$$ = new TrueCondition();}
|	'(' falsecond ')' {$$ = new FalseCondition();}
|	'(' ppos team UNUMSET INTEGER INTEGER region ')' {$$ = new PPosCondition($3,$4,$5,$6,$7);}
|	'(' bpos region ')' {$$ = new BPosCondition($3);}
|	'(' bowner team UNUMSET ')' {$$ = new BOwnerCondition($3,$4);}
|	'(' playm PLAYMODE ')' {$$ = new PlayMCondition($3);}
|	'(' unumcond UNDEFCV UNUMSET ')' {$$ = new UnumCondition($3,$4);}
|	condcomp
|	'(' andcond condlist ')' {$$ = new AndCondition($3);}
|	'(' orcond condlist ')' {$$ = new OrCondition($3);}
|	'(' notcond condition ')' {$$ = new NotCondition($3);}
|	DEFCS	{
			if (!rulebase->IsCondition($1))
			{
				char estr[255];
				sprintf(estr,"%s is not defined as a condition", $1->GetName());
				yyerror(estr);
				YYERROR; 
			}
			$$ = new VarCondition($1);
		}
;



condcomp:
	'(' timecond ccop INTEGER ')' {$$ = new CondComp(svtime,0,$3,constant,$4);}
|	'(' INTEGER ccop timecond ')' {$$ = new CondComp(constant,$2,$3,svtime,0);}
|	'(' opp_goals ccop INTEGER ')' {$$ = new CondComp(oppgoals,0,$3,constant,$4);}
|	'(' INTEGER ccop opp_goals ')' {$$ = new CondComp(constant,$2,$3,oppgoals,0);}
|	'(' our_goals ccop INTEGER ')' {$$ = new CondComp(ourgoals,0,$3,constant,$4);}
|	'(' INTEGER ccop our_goals ')' {$$ = new CondComp(constant,$2,$3,ourgoals,0);}
|	'(' goal_diff ccop INTEGER ')' {$$ = new CondComp(goaldiff,0,$3,constant,$4);}
|	'(' INTEGER ccop goal_diff ')' {$$ = new CondComp(constant,$2,$3,goaldiff,0);}

;


ccop:
	'=' '='	{$$ = eq;}
|	'<'	{$$ = lt;}
|	'<' '='	{$$ = lte;}
|	'>'	{$$ = gt;}
|	'>' '='	{$$ = gte;}
|	'!' '='	{$$ = neq;}
;


condlist:
	condition	{$$ = new conditionList(); $$->next = NULL; $$->cond = $1;}
|	condition condlist {$$ = new conditionList(); $$->next = $2; $$->cond = $1;}
;


// All action related stuff below

action:
	'(' pos region ')' {$$ = new StdAction(POS,$3,0,NULL);}
|	'(' home region ')' {$$ = new StdAction(HOME,$3,0,NULL);}
|	'(' mark UNUMSET ')' {$$ = new StdAction(MARK,NULL,0,$3);}
|	'(' markl UNUMSET ')' {$$ = new StdAction(MARKLUNUM,NULL,0,$3);}
|	'(' markl region ')' {$$ = new StdAction(MARKLREG,$3,0,NULL);}
|	'(' oline region ')' {$$ = new StdAction(OLINE,$3,0,NULL);}
|	'(' htype INTEGER ')' {$$ = new StdAction(HTYPE,NULL,$3,NULL);}
|	'(' pass region ')' {$$ = new StdAction(PASSREG,$3,0,NULL);}
|	'(' pass UNUMSET ')' {$$ = new StdAction(PASSUNUM, NULL,0,$3);}
|	'(' dribble region ')' {$$ = new StdAction(DRIBBLE,$3,0,NULL);}
|	'(' clearact region ')' {$$ = new StdAction(CLEAR,$3,0,NULL);}
|	'(' shoot ')' {$$ = new StdAction(SHOOT,NULL,0,NULL);}
|	'(' hold ')' {$$ = new StdAction(HOLD,NULL,0,NULL);}
|	'(' intercept ')' {$$ = new StdAction(INTERCEPT,NULL,0,NULL);}
|	'(' tackle UNUMSET ')' {$$ = new StdAction(TACKLE,NULL,0,$3);}
|	DEFCS	{
			if (!rulebase->IsAction($1))
			{
				char estr[255];
				sprintf(estr,"%s is not defined as a action", $1->GetName());
				yyerror(estr);
				YYERROR; 
			}
			$$ = new VarAction($1);
		}
;

// All Region related grammar rules are below

region:		
	point {$$ = (Region *)$1;}
|	'(' rec point point ')'		{$$ = new RectRegion($3,$4);}
|	'(' tri point point point ')'		{$$ = new TriRegion($3,$4,$5);}
|	'(' regnull ')'	{$$ = new NullRegion;}
|	'(' ureg reglist ')' {$$ = new UnionRegion($3);}
|	'(' arc point NUMBER NUMBER NUMBER NUMBER ')' {$$ = new ArcRegion($3, $4, $5, $6, $7);}
|	DEFCS {
			if (!rulebase->IsRegion($1))
			{
				char estr[255];
				sprintf(estr,"%s is not defined as a region", $1->GetName());
				yyerror(estr);
				YYERROR; 
			}
			$$ = new VarRegion($1);
		}
;


point:
	'(' pt NUMBER NUMBER ')'	{$$ = new PointRegion($3,$4);}
|	'(' pt ball ')'		{$$ = new PointRegion(our,0);}
|	'(' pt team INTEGER ')'	{$$ = new PointRegion($3,$4);}
|	'(' pointarith ')'		{$$ = $2;}
|	'(' pt team UNDEFCV ')'	{$$ = new UnumSetPoint($3,$4);}
|	DEFCS	{
			if (!rulebase->IsPointRegion($1))
			{
				char estr[255];
				sprintf(estr,"%s is not defined as a point", $1->GetName());
				yyerror(estr);
				YYERROR; 
			}
			$$ = new VarPtRegion($1);
		}
;




pointarith:
	pointarith '+' pointarith	{$$ = new PointArithRegion($1,$3,PTADD);}
|	pointarith '-' pointarith	{$$ = new PointArithRegion($1,$3,PTSUBTRACT);}
|	pointarith '*' pointarith	{$$ = new PointArithRegion($1,$3,PTMULTIPLY);}
|	pointarith '/' pointarith	{$$ = new PointArithRegion($1,$3,PTDIVIDE);}
|	pt NUMBER NUMBER point		{$$ = new PointArithRegion(new PointRegion($2, $3), $4, PTADD);}
|	point
;

reglist:
	region	{$$ = new regionList; $$->next = NULL; $$->reg = $1;}
	| region reglist {$$=new regionList; $$->next = $2; $$->reg = $1;}
;




// Some basic rules are below

team:
	oppteam		{$$ = opp;}
|	ourteam		{$$ = our;}
;

	


NUMBER:
	DOUBLE 
	| INTEGER {$$ = (double)$1;}
;



UNUMSET:
	'{' UNUMSET_WO_BRACES '}' {$$ = $2;}
;

UNUMSET_WO_BRACES:
	INTEGER				{$$ = new UnumSet(); $$->Set($1);}
|	INTEGER UNUMSET_WO_BRACES	{$$ = $2; $$->Set($1);}
|	UNDEFCV				{$$ = new UnumSet(); $$->SetCLangVar($1);}
|	UNDEFCV UNUMSET_WO_BRACES	{$$ = $2; $$->SetCLangVar($1);}
;



%%



bool isWhitespace(char c)
{
	return (c == ' ') || (c == '\t') || (c == '\r') || (c == '\n');
}

bool isDouble(char *c)
{
	char *ep;
	strtof(c,&ep);
	if (ep > c && isEndOfToken(*ep))
		return true;
	return false;
}

bool isInteger(char *c)
{
	char *ep;
	strtol(c,&ep,10);
	if (ep > c && isEndOfToken(*ep))
		return true;
	return false;

}

bool isEndOfToken(char c)
{
        return isWhitespace(c) || c == ')' || c == '}' || c=='(' || c == '{' || c == '\0';
}

int yylex(void *lvalpt, void *ctrlp)
{
	parsestruct *ctrl = (parsestruct *)ctrlp;
#ifdef CL_LEX_DEBUG
	//cout << "'" << ctrl->str[ctrl->off] << "'" << endl;
	cout << "'";
	for (int i=lastoff;i<ctrl->off;i++)
		cout << ctrl->str[i];
	cout << "'" << endl;
	lastoff = ctrl->off;
#endif
        //cout << &ctrl->str[ctrl->off] << endl;
	YYSTYPE *lvalp = (YYSTYPE *)lvalpt;
	// skip whitespace
	while (isWhitespace(ctrl->str[ctrl->off]))
		ctrl->off++;
	if (ctrl->str[ctrl->off] == '\"')
	{
		int cso = 1;
		char cs[255];
		cs[0] = '\"';
		ctrl->off++;
		while (ctrl->str[ctrl->off] != '\"')
		{
			cs[cso++] = ctrl->str[ctrl->off++];
		}
		cs[cso] = '\"';
		cso++;
		ctrl->off++;
		cs[cso] = '\0';
		lvalp->clv = new CLangVar(cs);
		
		if (rulebase->IsDefinedStr(lvalp->clv))
			return DEFCS;
		else
			return UNDEFCS;
		
	}
	else if (ctrl->str[ctrl->off] == '(')
	{
		ctrl->off++;
		
		return '(';
	}
	else if (ctrl->str[ctrl->off] == ')')
	{
		
		ctrl->off++;
		return ')';
	}
	else if (ctrl->str[ctrl->off] == '{')
	{
		ctrl->off++;
		return '{';
	}
	else if (ctrl->str[ctrl->off] == '}')
	{
		ctrl->off++;
		return '}';
	}
	else if (ctrl->str[ctrl->off] == '=')
	{
		ctrl->off++;
		return '=';
	}
	else if (ctrl->str[ctrl->off] == '|')
	{
		ctrl->off++;
		return '|';
	}
	else if (ctrl->str[ctrl->off] == '>')
	{
		ctrl->off++;
		return '>';
	}
	else if (ctrl->str[ctrl->off] == '<')
	{
		ctrl->off++;
		return '<';
	}
	else if (ctrl->str[ctrl->off] == '+' && isEndOfToken(ctrl->str[ctrl->off+1]))
	{
		ctrl->off++;
		return '+';
	}
	else if (ctrl->str[ctrl->off] == '-' && isEndOfToken(ctrl->str[ctrl->off+1]))
	{
		ctrl->off++;
		return '-';
	}
	else if (ctrl->str[ctrl->off] == '*')
	{
		ctrl->off++;
		return '*';
	}
	else if (ctrl->str[ctrl->off] == '/')
	{
		ctrl->off++;
		return '/';
	}
	else if (ctrl->str[ctrl->off] == '!')
	{
		ctrl->off++;
		return '!';
	}
	
	else if (strncmp(&ctrl->str[ctrl->off],"pt",2) == 0 && isEndOfToken(ctrl->str[ctrl->off+2]))
	{

		ctrl->off += 2;
		return pt;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"ball",4) == 0  && isEndOfToken(ctrl->str[ctrl->off+4]))
	{
		ctrl->off += 4;		
		return ball;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"opp",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{
		ctrl->off += 3;
		return oppteam;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"our",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{
		ctrl->off += 3;
		return ourteam;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"rec",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{
		ctrl->off += 3;
		return rec;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"null",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{
		ctrl->off += 4;
		return regnull;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"arc",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{
		ctrl->off += 3;
		return arc;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"reg",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{

		ctrl->off += 3;
		return ureg;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"tri",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{

		ctrl->off += 3;
		return tri;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"pos",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{

		ctrl->off += 3;
		return pos;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"markl",5) == 0 && isEndOfToken(ctrl->str[ctrl->off+5]))
	{

		ctrl->off += 5;
		return markl;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"home",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{

		ctrl->off += 4;
		return home;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"mark",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{

		ctrl->off += 4;
		return mark;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"oline",5) == 0 && isEndOfToken(ctrl->str[ctrl->off+5]))
	{

		ctrl->off += 5;
		return oline;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"htype",5) == 0 && isEndOfToken(ctrl->str[ctrl->off+5]))
	{

		ctrl->off += 5;
		return htype;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"pass",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{

		ctrl->off += 4;
		return pass;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"dribble",7) == 0 && isEndOfToken(ctrl->str[ctrl->off+7]))
	{

		ctrl->off += 7;
		return dribble;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"clear",5) == 0 && isEndOfToken(ctrl->str[ctrl->off+5]))
	{

		ctrl->off += 5;
		return clearact;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"shoot",5) == 0 && isEndOfToken(ctrl->str[ctrl->off+5]))
	{

		ctrl->off += 5;
		return shoot;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"hold",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{

		ctrl->off += 4;
		return hold;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"intercept",9) == 0 && isEndOfToken(ctrl->str[ctrl->off+9]))
	{

		ctrl->off += 9;
		return intercept;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"tackle",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{

		ctrl->off += 6;
		return tackle;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"true",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{

		ctrl->off += 4;
		return truecond;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"false",5) == 0 && isEndOfToken(ctrl->str[ctrl->off+5]))
	{

		ctrl->off += 5;
		return falsecond;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"ppos",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{

		ctrl->off += 4;
		return ppos;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"bpos",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{

		ctrl->off += 4;
		return bpos;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"playm",5) == 0 && isEndOfToken(ctrl->str[ctrl->off+5]))
	{

		ctrl->off += 5;
		return playm;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"bowner",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{

		ctrl->off += 6;
		return bowner;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"and",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{

		ctrl->off += 3;
		return andcond;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"or",2) == 0 && isEndOfToken(ctrl->str[ctrl->off+2]))
	{

		ctrl->off += 2;
		return orcond;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"not",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{

		ctrl->off += 3;
		return notcond;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"unum",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{

		ctrl->off += 4;
		return unumcond;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"time_over",9) == 0 && isEndOfToken(ctrl->str[ctrl->off+9]))
	{
		lvalp->md = time_over;
		ctrl->off += 9;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"time",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{

		ctrl->off += 4;
		return timecond;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"opp_goals",9) == 0 && isEndOfToken(ctrl->str[ctrl->off+9]))
	{

		ctrl->off += 9;
		return opp_goals;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"our_goals",9) == 0 && isEndOfToken(ctrl->str[ctrl->off+9]))
	{

		ctrl->off += 9;
		return our_goals;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"goal_diff",9) == 0 && isEndOfToken(ctrl->str[ctrl->off+9]))
	{

		ctrl->off += 9;
		return goal_diff;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"play_on",7) == 0 && isEndOfToken(ctrl->str[ctrl->off+7]))
	{
		lvalp->md = play_on;
		ctrl->off += 7;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"bko",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{
		lvalp->md = bko;
		ctrl->off += 3;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"ko_our",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = ko_our;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"ko_opp",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = ko_opp;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"ki_our",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = ki_our;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"ki_opp",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = ki_opp;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"fk_our",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = fk_our;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"fk_opp",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = fk_opp;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"ck_our",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = ck_our;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"ck_opp",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = ck_opp;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"gc_our",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = gc_our;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"gc_opp",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = gc_opp;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"ag_our",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = ag_our;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"ag_opp",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = ag_opp;
		ctrl->off += 6;
		return PLAYMODE;
	}
        else if (strncmp(&ctrl->str[ctrl->off],"gk_our",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = gk_our;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"gk_opp",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		lvalp->md = gk_opp;
		ctrl->off += 6;
		return PLAYMODE;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"dont",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{

		ctrl->off += 4;
		return dirdont;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"do",2) == 0 && isEndOfToken(ctrl->str[ctrl->off+2]))
	{

		ctrl->off += 2;
		return dirdo;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"model",5) == 0 && isEndOfToken(ctrl->str[ctrl->off+5]))
	{

		ctrl->off += 5;
		return model;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"direc",5) == 0 && isEndOfToken(ctrl->str[ctrl->off+5]))
	{

		ctrl->off += 5;
		return direc;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"definerule",10) == 0 && isEndOfToken(ctrl->str[ctrl->off+10]))
	{

		ctrl->off += 10;
		return definerule;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"definer",7) == 0 && isEndOfToken(ctrl->str[ctrl->off+7]))
	{

		ctrl->off += 7;
		return definer;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"definea",7) == 0 && isEndOfToken(ctrl->str[ctrl->off+7]))
	{

		ctrl->off += 7;
		return definea;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"defined",7) == 0 && isEndOfToken(ctrl->str[ctrl->off+7]))
	{
		
		ctrl->off += 7;
		return definedir;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"definec",7) == 0 && isEndOfToken(ctrl->str[ctrl->off+7]))
	{
		
		ctrl->off += 7;
		return definec;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"define",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		
		ctrl->off += 7;
		return define;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"delete",6) == 0 && isEndOfToken(ctrl->str[ctrl->off+6]))
	{
		
		ctrl->off += 6;
		return deleterule;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"rule",4) == 0 && isEndOfToken(ctrl->str[ctrl->off+4]))
	{

		ctrl->off += 4;
		return ruletoken;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"on",2) == 0 && isEndOfToken(ctrl->str[ctrl->off+2]))
	{

		ctrl->off += 2;
		return ruleon;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"off",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{

		ctrl->off += 3;
		return ruleoff;
	}
	else if (strncmp(&ctrl->str[ctrl->off],"all",3) == 0 && isEndOfToken(ctrl->str[ctrl->off+3]))
	{

		ctrl->off += 3;
		return allrules;
	}
	else if (isInteger(&ctrl->str[ctrl->off]))
	{
		char *ep;
		lvalp->integer = strtol(&ctrl->str[ctrl->off],&ep,10);

		ctrl->off = (ep - ctrl->str);
		return INTEGER;
	}
	else if (isDouble(&ctrl->str[ctrl->off]))
	{
		char *ep;
		lvalp->real = strtof(&ctrl->str[ctrl->off],&ep);
		
		ctrl->off = (ep - ctrl->str);
		return DOUBLE;		
	}
	else
	{
		char cs[255];
		int cso = 0;
		while (!isEndOfToken(ctrl->str[ctrl->off]))
		{
			cs[cso++] = ctrl->str[ctrl->off++];
		}
		cs[cso] = '\0';
		lvalp->clv = new CLangVar(cs);
		if (rulebase->IsDefinedVar(lvalp->clv))
			return DEFCV;
		else
			return UNDEFCV;

	}
}

int yyerror(char *s)
{
	char *c = strchr(s,',');
	if (c)
		*c = ':';
	cerr << "CLang " << s << endl;
	cout << "CLang " << s << endl;
	cout << "err" << endl;
	return 0;
}

