/*
Copyright (c) 2003-2004, Gregory Kuhlmann, Peter Stone, Justin Lallinger,
University of Texas at Austin
All right reserved

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the University of Texas nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _CLANG_TYPES_
#define _CLANG_TYPES_

#include <iostream>

using namespace std;

class CLangVar
{
 public:
  CLangVar();
  CLangVar( char *a );
  CLangVar & operator =( char *a );
  bool operator ==( CLangVar & v );
  void Set( char *a );
  void Output( ostream & os );
  const char *GetName() { return cvar; }

 private:
  char cvar[255];
} ;

enum CLPlayMode
{
  bko = 0, time_over, play_on, ko_our, ko_opp, ki_our, ki_opp,
  fk_our, fk_opp, ck_our, ck_opp, gk_our, gk_opp, gc_our,
  gc_opp, ag_our, ag_opp
} ;

enum CLTeam
{
  our = 0,
  opp = 1
} ;

/// The UnumSet class handles all the CLang unumset constucts
class UnumSet
{
 public:
  UnumSet()
  {
    set = 0;
    inst = 0;
    v = NULL;
  }

  void Clear();
  void Clear( int plr );

  void Set( int plr );
  bool IsSet( int plr, bool checkinst = true );
  void Output( ostream & os, bool dispvariable = true );
  void Instantiate( int plr );
  void ClearInstantiation() { inst = 0; }
  bool ContainsVariable() { return ( v != NULL ); }
  void SetCLangVar( CLangVar * v ) { this->v = v; }
  CLangVar *GetCLangVar() { return v; }
  void InstantiateMinus( UnumSet * s );

 private:
  int set;
  int inst;
  CLangVar *v;
} ;

// These callback functions introduce a way for conditions to be
// evaluated and relative points to be calculated (ie, (pt ball))

extern bool (*CLGetPlayerKnown)(CLTeam t, int p);
extern double ( *CLGetPlayerX ) ( CLTeam t, int p );
extern double ( *CLGetPlayerY ) ( CLTeam t, int p );
extern int ( *CLGetBallOwnerUnum ) ();
extern CLTeam( *CLGetBallOwnerTeam ) ();
extern double ( *CLGetBallX ) ();
extern double ( *CLGetBallY ) ();
extern int ( *CLGetScore ) ( CLTeam t );
extern int ( *CLGetTime ) ();
extern CLPlayMode( *CLGetPlayMode ) ();
extern bool( *CLReverseCoords ) ();

class RuleBase;
extern RuleBase *rulebase;

#endif
